package sun.nio.ch;

import java.net.StandardSocketOptions;
import java.net.SocketException;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.channels.IllegalBlockingModeException;
import java.nio.channels.ClosedChannelException;
import java.net.SocketTimeoutException;
import java.net.Socket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.io.IOException;
import java.net.ServerSocket;

public class ServerSocketAdaptor extends ServerSocket
{
    private final ServerSocketChannelImpl ssc;
    private volatile int timeout;
    
    public static ServerSocket create(ServerSocketChannelImpl serverSocketChannelImpl) {
        try {
            return new ServerSocketAdaptor(serverSocketChannelImpl);
        }
        catch (IOException cause) {
            throw new Error(cause);
        }
    }
    
    private ServerSocketAdaptor(ServerSocketChannelImpl ssc) throws IOException {
        this.timeout = 0;
        this.ssc = ssc;
    }
    
    @Override
    public void bind(SocketAddress socketAddress) throws IOException {
        this.bind(socketAddress, 50);
    }
    
    @Override
    public void bind(SocketAddress socketAddress, int n) throws IOException {
        if (socketAddress == null) {
            socketAddress = new InetSocketAddress(0);
        }
        try {
            this.ssc.bind(socketAddress, n);
        }
        catch (Exception ex) {
            Net.translateException(ex);
        }
    }
    
    @Override
    public InetAddress getInetAddress() {
        if (!this.ssc.isBound()) {
            return null;
        }
        return Net.getRevealedLocalAddress(this.ssc.localAddress()).getAddress();
    }
    
    @Override
    public int getLocalPort() {
        if (!this.ssc.isBound()) {
            return -1;
        }
        return Net.asInetSocketAddress(this.ssc.localAddress()).getPort();
    }
    
    @Override
    public Socket accept() throws IOException {
        synchronized (this.ssc.blockingLock()) {
            if (!this.ssc.isBound()) {
                throw new IllegalBlockingModeException();
            }
            try {
                if (this.timeout == 0) {
                    SocketChannel accept3 = this.ssc.accept();
                    if (accept3 == null && !this.ssc.isBlocking()) {
                        throw new IllegalBlockingModeException();
                    }
                    return accept3.socket();
                }
                else {
                    this.ssc.configureBlocking(false);
                    try {
                        SocketChannel accept;
                        if ((accept = this.ssc.accept()) != null) {
                            Socket socket2 = accept.socket();
                            if (this.ssc.isOpen()) {
                                this.ssc.configureBlocking(true);
                            }
                            return socket2;
                        }
                        else {
                            long n = (long)this.timeout;
                            while (this.ssc.isOpen()) {
                                long currentTimeMillis = System.currentTimeMillis();
                                SocketChannel accept2;
                                if (this.ssc.poll((int)Net.POLLIN, n) > 0 && (accept2 = this.ssc.accept()) != null) {
                                    Socket socket = accept2.socket();
                                    if (this.ssc.isOpen()) {
                                        this.ssc.configureBlocking(true);
                                    }
                                    return socket;
                                }
                                else {
                                    n -= System.currentTimeMillis() - currentTimeMillis;
                                    if (n <= 0L) {
                                        throw new SocketTimeoutException();
                                    }
                                    continue;
                                }
                            }
                            throw new ClosedChannelException();
                        }
                    }
                    finally {
                        if (this.ssc.isOpen()) {
                            this.ssc.configureBlocking(true);
                        }
                    }
                }
            }
            catch (Exception ex) {
                Net.translateException(ex);
                assert false;
                return null;
            }
        }
    }
    
    @Override
    public void close() throws IOException {
        this.ssc.close();
    }
    
    @Override
    public ServerSocketChannel getChannel() {
        return this.ssc;
    }
    
    @Override
    public boolean isBound() {
        return this.ssc.isBound();
    }
    
    @Override
    public boolean isClosed() {
        return !this.ssc.isOpen();
    }
    
    @Override
    public void setSoTimeout(int timeout) throws SocketException {
        this.timeout = timeout;
    }
    
    @Override
    public int getSoTimeout() throws SocketException {
        return this.timeout;
    }
    
    @Override
    public void setReuseAddress(boolean b) throws SocketException {
        try {
            this.ssc.setOption(StandardSocketOptions.SO_REUSEADDR, Boolean.valueOf(b));
        }
        catch (IOException ex) {
            Net.translateToSocketException(ex);
        }
    }
    
    @Override
    public boolean getReuseAddress() throws SocketException {
        try {
            return (boolean)this.ssc.getOption(StandardSocketOptions.SO_REUSEADDR);
        }
        catch (IOException ex) {
            Net.translateToSocketException(ex);
            return false;
        }
    }
    
    @Override
    public String toString() {
        if (!this.isBound()) {
            return "ServerSocket[unbound]";
        }
        return "ServerSocket[addr=" + this.getInetAddress() + ",localport=" + this.getLocalPort() + "]";
    }
    
    @Override
    public void setReceiveBufferSize(int i) throws SocketException {
        if (i <= 0) {
            throw new IllegalArgumentException("size cannot be 0 or negative");
        }
        try {
            this.ssc.setOption(StandardSocketOptions.SO_RCVBUF, Integer.valueOf(i));
        }
        catch (IOException ex) {
            Net.translateToSocketException(ex);
        }
        return;
    }
    
    @Override
    public int getReceiveBufferSize() throws SocketException {
        try {
            return (int)this.ssc.getOption(StandardSocketOptions.SO_RCVBUF);
        }
        catch (IOException ex) {
            Net.translateToSocketException(ex);
            return -1;
        }
    }
}
